#include<iostream>
#include<vector>
using namespace std;

#include "burgers.h"

#include "dco.hpp"
typedef double DCO_BT;
typedef dco::ga1s<DCO_BT> DCO_AM;
typedef DCO_AM::type DCO_AT;
typedef DCO_AM::tape_t DCO_TAPE_T;

#include <omp.h>

int main() {
  omp_set_num_threads(NT);
  vector<DCO_AT> y(n,0);
  for (int i=1;i<n-1;i++) y[i]=sin(2*PI*i/n);
  DCO_AM::global_tape=DCO_TAPE_T::create();
  DCO_AM::global_tape->register_variable(y);
  vector<DCO_AT> yc(y);
  burgers(yc);  
  for(int i=n/2-bw;i<n/2+bw;i++) 
    DCO_AM::global_tape->register_output_variable(yc[i]);
  vector<vector<double>> M_dydy(2*bw,vector<double>(n));
#pragma omp parallel
  {
    int tid=omp_get_thread_num();
    assert(NT==omp_get_max_threads());
    dco::adjoint_vector<DCO_TAPE_T,double,2*bw/NT> av(DCO_AM::global_tape);
    for(int i=n/2-bw,j=0;i<n/2+bw;i++) {
      if(i%NT!=tid) continue;
      av.derivative(yc[i])[j]=1;
      j++;
    }
    av.interpret_adjoint();
    for(int i=n/2-bw,k=0;i<n/2+bw;i++) {
      if(i%NT!=tid) continue;
      for(int j=1;j<n-1;j++) 
        M_dydy[i-n/2+bw][j]=av.derivative(y[j])[k];
      k++;
    }
  }
  DCO_TAPE_T::remove(DCO_AM::global_tape);
  for(int i=n/2-bw;i<n/2+bw;i++) 
    for(int j=1;j<n-1;j++)
      cout << i << " " << j << ": " << M_dydy[i-n/2+bw][j] << endl;
  return 0;
}

