#include <cmath>
#include <vector>
#include <iostream>
using namespace std;

#include "dco.hpp"
#include "libor.h"

template <typename T>
void compute_hessian(const vector<T> &pL, vector<vector<T> >& ddPdLL) {

  typedef dco::ga1s<typename dco::gt1s<T>::type> DCO_M;
  typedef typename DCO_M::type DCO_T;
  typedef typename DCO_M::tape_t DCO_TAPE_T;

  vector<DCO_T> L; L.assign(begin(pL), end(pL));
  DCO_T P=0;

  srand(0); default_random_engine generator(0);
  normal_distribution<double> distribution(0.0,1.0);
  vector<vector<double>> Z(p,vector<double>(m));
  for (int j=0; j<p;j++)
    for (int i=0;i<m;i++)
      Z[j][i]=0.3+distribution(generator);


  DCO_M::global_tape=DCO_TAPE_T::create();
  DCO_M::global_tape->register_variable(L);
  typename DCO_TAPE_T::position_t tpos=DCO_M::global_tape->get_position();
  for(int j=0;j<n;j++) {
    dco::derivative(dco::value(L[j]))=1;
    libor(L,P,Z);
    DCO_M::global_tape->register_output_variable(P);
    dco::value(dco::derivative(P))=1;
    DCO_M::global_tape->interpret_adjoint_to(tpos);
    for(int i=0;i<n;i++) {
      ddPdLL[i][j]=dco::derivative(dco::derivative(L[i]));
      dco::derivative(L[i])=0; 
    }
    dco::derivative(dco::value(L[j]))=0;
    DCO_M::global_tape->reset_to(tpos);
  }
  DCO_TAPE_T::remove(DCO_M::global_tape);
}

int main() {
  typedef dco::ga1s<double> DCO_M;
  typedef DCO_M::type DCO_T;
  typedef DCO_M::tape_t DCO_TAPE_T;

  vector<DCO_T> L(n,0.05);
  vector<vector<DCO_T> > H(n,vector<DCO_T>(n,0));

  DCO_M::global_tape = DCO_TAPE_T::create();
  DCO_M::global_tape->register_variable(L);
  
  compute_hessian(L, H);

  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      derivative(H[i][j]) = 1.0;

  DCO_M::global_tape->interpret_adjoint();
  
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      cout << "ddP/dL[" << i << "]dL[" << j << "]=" << value(H[i][j]) << endl;

  for (int k=0;k<n;k++)
    cout << "sum_ij d^3P/dL[i]dL[j]dL[" << k << "]=" << derivative(L[k]) << endl;
  return 0;
}
